#ifndef COMPARABLE_HPP
#define COMPARABLE_HPP

template<class T, typename Type = double>
struct Comparable
{
    static_assert(std::is_default_constructible_v<Type>,
        "Second argument type mus be default constructible");

    Comparable() = default;
    Comparable(const Type& value)
        : value_(value)
    {
    }

    const auto& value() const { return value_; }
    auto& value() { return value_; }

private:
    Type value_;
};

template<class T, typename Type>
bool operator==(const Comparable<T, Type>& lhs, const Comparable<T, Type>& rhs) { return lhs.value() == rhs.value(); }
template<class T, typename Type>
bool operator!=(const Comparable<T, Type>& lhs, const Comparable<T, Type>& rhs) { return lhs.value() != rhs.value(); }
template<class T, typename Type>
bool operator> (const Comparable<T, Type>& lhs, const Comparable<T, Type>& rhs) { return lhs.value() > rhs.value(); }
template<class T, typename Type>
bool operator>=(const Comparable<T, Type>& lhs, const Comparable<T, Type>& rhs) { return lhs.value() >= rhs.value(); }
template<class T, typename Type>
bool operator< (const Comparable<T, Type>& lhs, const Comparable<T, Type>& rhs) { return lhs.value() < rhs.value(); }
template<class T, typename Type>
bool operator<=(const Comparable<T, Type>& lhs, const Comparable<T, Type>& rhs) { return lhs.value() <= rhs.value(); }

#endif // COMPARABLE_HPP
