#ifndef DISCOUNT_HPP
#define DISCOUNT_HPP

#include <vector>
#include <memory>
#include <utility>

#include "product.hpp"

class DiscountVisitor;
class Discount;
using DiscountPtr = std::shared_ptr<Discount>;

class Discount
{
public:
    virtual void accept(DiscountVisitor&) = 0;
    virtual DiscountPtr copy() const = 0;

    Price calculatePrice(const Products& products) const;

    virtual Products calculateNewPrices(const Products& products) const = 0;

    virtual ~Discount() = default;
};

template<typename T, typename ... Args>
std::shared_ptr<T> makeDiscount(Args... args)
{
    static_assert(std::is_base_of_v<Discount, T>, "Type T must be derived from Discount type" );
    return std::make_shared<T>(args...);
}

class Discounts
{
public:
    Discounts();

    auto begin() const { return discounts_.begin(); }
    auto end() const { return discounts_.end(); }

    void add(DiscountPtr discount) { discounts_.push_back(discount); }

private:
    using Container = std::vector<std::shared_ptr<Discount>>;

    Container discounts_;
};

#endif // DISCOUNT_HPP