#include "discount_reader.hpp"

#include "discounts/no_discount.hpp"
#include "discounts/percent_discount.hpp"
#include "discounts/one_of_Nth_free_discount.hpp"
#include "discounts/complex_discount.hpp"

DiscountFile::DiscountFile(const FilePath &path)
    : file_(path)
{
    if (!file_.is_open())
        throw "NI";
}

DiscountFile::operator bool()
{
    return !file_.eof();
}

DiscountReader::DiscountReader(DiscountFile& file)
    : file_(file)
{
    prototypes_.emplace("NoDiscount", makeDiscount<NoDiscount>());
    prototypes_.emplace("PercentDiscount", makeDiscount<PercentDiscount>());
    prototypes_.emplace("OneOfNthFreeDiscount", makeDiscount<OneOfNthFreeDiscount>());
    prototypes_.emplace("ComplexDiscount", makeDiscount<ComplexDiscount>());
}

DiscountPtr DiscountReader::read()
{
    std::string type;

    file_ >> type;

    auto prototype = getPrototype(type);

    prototype->accept(*this);

    return prototype;
}

DiscountPtr DiscountReader::getPrototype(std::string type)
{
    auto it = prototypes_.find(type);

    if (it == prototypes_.end())
    {
        throw "NI";
    }

    return it->second->copy();
}

void DiscountReader::visit(NoDiscount&)
{
}

void DiscountReader::visit(PercentDiscount& discount)
{
    double value;
    file_ >> value;
    discount = PercentDiscount(value);
}

void DiscountReader::visit(OneOfNthFreeDiscount& discount)
{
    unsigned int value;
    file_ >> value;
    discount = OneOfNthFreeDiscount(value);
}

void DiscountReader::visit(ComplexDiscount& discount)
{
    auto first = read();
    auto second = read();

    discount = ComplexDiscount(first, second);
}

DiscountFile& operator>>(DiscountFile& file, DiscountPtr& discount)
{
    discount = DiscountReader(file).read();
    return file;
}