#ifndef DISCOUNT_READER_HPP
#define DISCOUNT_READER_HPP

#include <filesystem>
#include <fstream>
#include <map>

#include "discount_visitor.hpp"
#include "discount.hpp"

using FilePath = std::filesystem::path;

class DiscountFile
{
public:
    DiscountFile(const FilePath&);

    template<typename T>
    DiscountFile& operator>>(T& obj)
    {
        file_ >> obj;
        return *this;
    }

    operator bool();

private:
    std::ifstream file_;
};

class DiscountReader : public DiscountVisitor
{
public:
    DiscountReader(DiscountFile&);

    DiscountPtr read();

private:
    void visit(NoDiscount&) override;
    void visit(PercentDiscount&) override;
    void visit(OneOfNthFreeDiscount&) override;
    void visit(ComplexDiscount&) override;

private:
    DiscountPtr getPrototype(std::string type);

    using Prototypes = std::map<std::string, DiscountPtr>;

    Prototypes prototypes_;

    DiscountFile& file_;
};


DiscountFile& operator>>(DiscountFile& file, DiscountPtr& discount);

#endif // DISCOUNT_READER_HPP