#include <cassert>
#include <algorithm>
#include <limits>

#include "one_of_Nth_free_discount.hpp"
#include "discount_visitor.hpp"

OneOfNthFreeDiscount::OneOfNthFreeDiscount(unsigned int value)
    : value_(value)
{
    assert(value_ >= 2);
}

void OneOfNthFreeDiscount::accept(DiscountVisitor& visitor)
{
    visitor.visit(*this);
}

DiscountPtr OneOfNthFreeDiscount::copy() const
{
    return makeDiscount< OneOfNthFreeDiscount>(value_);
}

Products OneOfNthFreeDiscount::calculateNewPrices(const Products & products) const
{
    Products result;

    int counter = 0;
    for (const auto& product : sort(products))
    {
        auto new_price = ++counter % value_ ? product.pricePerUnit() : Price(0);
        result.add(Product{ product.type(), product.count(), new_price });
    }

    return result;
}

Products OneOfNthFreeDiscount::sort(Products products) const
{
    std::sort(
        products.begin(),
        products.end(),
        [](const Product& lhs, const Product& rhs) {return lhs.price() < rhs.price(); });

    return products;
}
