#include <cassert>

#include "percent_discount.hpp"
#include "discount_visitor.hpp"

PercentDiscount::PercentDiscount(double value)
    : value_(value)
{
    assert(value_ >= 0 && value_ <= 1);
}

void PercentDiscount::accept(DiscountVisitor& visitor)
{
    visitor.visit(*this);
}

DiscountPtr PercentDiscount::copy() const
{
    return makeDiscount< PercentDiscount>(0);
}

Products PercentDiscount::calculateNewPrices(const Products & products) const
{
    Products result;

    for (const auto& product : products)
    {
        result.add(Product{ product.type(), product.count(), product.pricePerUnit() * (1.0 - value_) });
    }

    return result;
}
