#ifndef PRODUCT_HPP
#define PRODUCT_HPP

#include <vector>
#include <memory>
#include <string>

#include "comparable.hpp"

struct Price : public Comparable<Price, double>
{
    using Comparable<Price, double>::Comparable;
};

Price operator*(const Price& price, double scalar);
Price operator+(const Price& lhs, const Price& rhs);

struct Count : public Comparable<Count, double>
{
    using Comparable<Count, double>::Comparable;
};

Price operator*(const Price& price, const Count& count);


struct ProductType
{
    std::string type;
};

class Product
{
public:
    Product(ProductType, Count, Price);

    auto type() const { return type_; }
    auto count() const { return count_; }
    auto pricePerUnit() const { return price_per_unit_; }
    auto price() const { return price_per_unit_ * count_; }

private:
    ProductType type_;
    Count count_;
    Price price_per_unit_;
};

class Products
{
public:
    auto begin() const { return products_.begin(); }
    auto end() const { return products_.end(); }
    auto begin() { return products_.begin(); }
    auto end() { return products_.end(); }

    void add(const Product& product) { products_.push_back(product); }
private:
    using Container = std::vector<Product>;

    Container products_;
};

#endif // PRODUCT_HPP