#ifndef PRODUCT_SCANNER_HPP
#define PRODUCT_SCANNER_HPP

#include <memory>
#include <utility>

#include "product.hpp"

class ProductScanner
{
public:
    virtual Products scan() = 0;

    virtual ~ProductScanner() = default;
};

using ProductScannerPtr = std::shared_ptr<ProductScanner>;

template<typename T, typename ... Args>
std::shared_ptr<T> makeProductScanner(Args... args)
{
    static_assert(std::is_base_of_v<ProductScanner, T>, "Type T must be derived from ProductScanner type");
    return std::make_shared<T>(args...);
}

#endif // PRODUCT_SCANNER_HPP