#include "main_window.hpp"
#include <QPushButton>
#include <QFileDialog> 
#include <qmenubar.h>
#include <qmessagebox.h>

#include "qlist_discount_item.hpp"

#include "discounts_factory.hpp"
#include <discount_visitors/discount_name.hpp>

#include "discount_presenter.hpp"

#include <ui/ui_main_window.h>

MainWindow::MainWindow(QWidget* parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow())

{
    ui->setupUi(this);

    createActions();
}

void MainWindow::createActions()
{
    connect(ui->actionOpen, &QAction::triggered, this, &MainWindow::open);
    connect(ui->actionSave, &QAction::triggered, this, &MainWindow::save);
    connect(ui->actionQuit, &QAction::triggered, this, &MainWindow::quit);
    connect(ui->actionAbout, &QAction::triggered, this, &MainWindow::about);

    connect(ui->discountList, SIGNAL(itemDoubleClicked(QListWidgetItem*)), this, SLOT(listItemDoubleClicked(QListWidgetItem*)));
}

void MainWindow::setDiscounts(const Discounts & discounts)
{
    DiscountName discoutn_name;

    for (auto & discount : discounts)
    {
        setDiscount(discount);
    }
}

void MainWindow::setDiscount(const DiscountPtr & discount)
{
    DiscountName discoutn_name;

    auto qlist_item = new QListDiscountItem(discount, ui->discountList);

    discount->accept(discoutn_name);
    qlist_item->setText(QString::fromStdString(discoutn_name.getLastDiscountName()));

    ui->discountList->addItem(qlist_item);
}

void MainWindow::open()
{
    try
    {
        auto file_path = QFileDialog::getOpenFileName(this, tr("Open File"));

        if (file_path.isEmpty())
        {
            return;
        }

        auto discounts = DiscountsFactory().read(FilePath{ file_path.toStdWString() });

        setDiscounts(discounts);
    }
    catch (std::exception & e)
    {
        QMessageBox msgBox;
        msgBox.setText(QString("The document cannot be red\n reason:") + e.what());
        msgBox.exec();
    }
    catch (...)
    {
        QMessageBox msgBox;
        msgBox.setText(QString("The document cannot be red\n reason: unknown exeption"));
        msgBox.exec();
    }
}

void MainWindow::save()
{
}

void MainWindow::quit()
{
    close();
}

void MainWindow::about()
{
}

void MainWindow::listItemDoubleClicked(QListWidgetItem* item)
{
    auto discount_item = dynamic_cast<QListDiscountItem*>(item);

    assert(discount_item != nullptr);

    DiscountPresenter presenter;

    discount_item->discount()->accept(presenter);
}
